package classserver;

import java.io.*;
import java.net.*;

/**
 * Title:        Einfacher Class Server
 * Description:  Class Server zum Herunterladen und Verschieben von Class Dateien
 * Funktioniert wie ein HTTP Server
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * ClassServer ist eine abstrakte Klasse, welche die
 * grundlegende Funktionalitt eines WebServers definiert.
 * Beim Implementieren muss die <b>getBytes</b>
 * Methode definiert sein. Sie liest den Bytecode der Class-Datei
 *
 * Der ClassServer kreiert Threads und hrt auf GET Anfragen
 * Als Antwort liefert er die Class Datei, die im GET Header steht.
 *
 * @see ClassFileServer
 */
public abstract class ClassServer implements Runnable {

    private ServerSocket server = null;
    private int port;

    /**
     * Konstruktion des ClassServer:
     * er hrt an <b>Port</b> und liest die Class Datei
     * mit der Methode <b>getBytes</b>.
     *
     * @param port Portnummer
     * @exception IOException falls der Port besetzt ist
     */
    protected ClassServer(int port) throws IOException
    {
	this.port = port;
	server = new ServerSocket(port);
	newListener();
    }

    /**
     * liefert ein Bytearray mit der Classdatei
     *
     * @return Bytecode der Classdatei
     * @exception ClassNotFoundException Classdatei wurde nicht gefunden
     * @exception IOException falls ein fehler beim Lesen auftrat
     */
    public abstract byte[] getBytes(String path)
	throws IOException, ClassNotFoundException;

    /**
     * Hrt auf Anfragen, parsed den Header und sendet den Bytecode zurck
     */
    public void run()
    {
	Socket socket;

	// Verbindung akzeptieren
	try {
	    socket = server.accept();
	} catch (IOException e) {
	    System.out.println("Class Server ist tot: " + e.getMessage());
	    e.printStackTrace();
	    return;
	}

	// neuer Thread fr neue Anfrage
	newListener();

	try {
	    DataOutputStream out =
		new DataOutputStream(socket.getOutputStream());
	    try {
		// Pfad zur Classdatei
		BufferedReader in =
		    new BufferedReader(new InputStreamReader(socket.getInputStream()));
		String path = getPath(in);
		// Lesen des Bytecodes
		byte[] bytecodes = getBytes(path);
		// senden des Bytecodes
		try {
		    out.writeBytes("HTTP/1.0 200 OK\r\n");
		    out.writeBytes("Content-Length: " + bytecodes.length +
				   "\r\n");
		    out.writeBytes("Content-Type: application/java\r\n\r\n");
		    out.write(bytecodes);
		    out.flush();
		} catch (IOException ie) {
		    return;
		}

	    } catch (Exception e) {
		// Fehlercodes
		out.writeBytes("HTTP/1.0 400 " + e.getMessage() + "\r\n");
		out.writeBytes("Content-Type: text/html\r\n\r\n");
		out.flush();
	    }

	} catch (IOException ex) {
	    // Fehlerausgabe
	    System.out.println("Fehler beim Antworten: " + ex.getMessage());
	    ex.printStackTrace();

	} finally {
	    try {
              System.out.println("[ClassServer]socket.close()");
		socket.close();
	    } catch (IOException e) {
	    }
	}
    }

    /**
     * neuer ListenerThread
     */
    private void newListener()
    {
      System.out.println("[ClassServer]newListener Thread.start()");
	(new Thread(this)).start();
    }

    /**
     * Pfad zur Classdatei
     * aus dem HTTP Header
     */
    private static String getPath(BufferedReader in)
	throws IOException
    {
	String line = in.readLine();
	String path = "";

	// bestimme Klasse aus dem GET Header
	if (line.startsWith("GET /")) {
	    line = line.substring(5, line.length()-1).trim();

	    int index = line.indexOf(".class ");
	    if (index != -1) {
		path = line.substring(0, index).replace('/', '.');
	    }
	}

	// Rest des Headers
	do {
	    line = in.readLine();
	} while ((line.length() != 0) &&
	         (line.charAt(0) != '\r') && (line.charAt(0) != '\n'));

	if (path.length() != 0) {
          System.out.println("[ClassServer]getPath() :"+path);
	    return path;
	} else {
	    throw new IOException("Malformed Header");
	}
    }
}
